/**
 * Contains classes for reading and writing objects.
 */
package ru.amse.ksenofontova.jina.io;

import java.awt.Dimension;
import java.awt.Point;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import ru.amse.ksenofontova.jina.MainWindowParameters;

/**
 * Class for writing <code>MainWindowRarameters'</code> fields to specified XML-file
 * @author Eugenia
 *
 */
public class MainWindowParametersWriter implements MainWindowRarametersIOConstants {
    private final String myFileName;
    
    /**
     * Creates <code>MainWindowParametersWriter</code> for writing 
     * <code>MainWindowRarameters'</code> fields to specified XML-file.
     * @param fileName name of the given XML-file to write 
     * <code>MainWindowRarameters'</code> fields
     */
    public MainWindowParametersWriter (String fileName) {
        myFileName = fileName;
    }
    
    /**
     * Writes <code>MainWindowRarameters'</code> fields to XML-file specified 
     * in constructor
     * @throws SaveException when there are some problems with writing data
     * (not enought memory on disc)
     */
    public void write() throws SaveException {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory
                .newInstance();
            DocumentBuilder docBuilder = factory.newDocumentBuilder();
            Document doc = docBuilder.newDocument();
            Element rootElement = doc.createElement(MAIN_WINDOW_PARAMETERS_TAG_NAME);
            
            if(MainWindowParameters.Size != MainWindowParameters.DEFAULT_SIZE) {                
                Element sizeElement = doc.createElement(SIZE_TAG_NAME);
                writeSize(sizeElement, MainWindowParameters.Size);
                rootElement.appendChild(sizeElement);
            }
            
            if(MainWindowParameters.Location != MainWindowParameters.DEFAULT_LOCATION) {
                Element locationElement = doc.createElement(LOCATION_TAG_NAME);
                writeLocation(locationElement, MainWindowParameters.Location);
                rootElement.appendChild(locationElement);
            }            
                        
            doc.appendChild(rootElement);
            TransformerFactory tFactory = TransformerFactory.newInstance();
            tFactory.setAttribute("indent-number", new Integer(4));
            Transformer transformer = tFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.transform(new DOMSource(doc), new StreamResult(new OutputStreamWriter(
                new FileOutputStream(new File(myFileName)))));
        } catch( Throwable t ) {
            throw new SaveException(t.getMessage());
        }
        
    }

    /**
     * Writes given <code>Dimension</code> object which is to be interpreted 
     * as size to the given <code>Element</code>
     * @param sizeElement the <code>Element</code> to write Font to
     * @param taskFont the given Font to write
     */
    private void writeSize(Element sizeElement, Dimension size) {
        sizeElement.setAttribute(WIDTH_TAG_NAME, size.width + "");
        sizeElement.setAttribute(HEIGHT_TAG_NAME, size.height + "");
    }
    
    

    /**
     * Writes given <code>Point</code> object which is to be interpreted 
     * as location to the given <code>Element</code>
     * @param locationElement the <code>Element</code> to write Color to
     * @param location the given Location to write
     */
    private void writeLocation(Element locationElement, Point location) {
        locationElement.setAttribute(X_POSITION_TAG_NAME, location.x + "");
        locationElement.setAttribute(Y_POSITION_TAG_NAME, location.y + ""); 
    }

}
