/**
 * Contains classes for reading and writing objects.
 */
package ru.amse.ksenofontova.jina.io;

import java.io.File;
import java.text.ParseException;
import java.util.Date;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Attr;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.model.ExecutionState;
import ru.amse.ksenofontova.jina.model.IRepositopyManager;
import ru.amse.ksenofontova.jina.model.ITask;
import ru.amse.ksenofontova.jina.model.Identity;
import ru.amse.ksenofontova.jina.model.Priority;
import ru.amse.ksenofontova.jina.model.TimeLimitation;

/**
 * Class for reading tasks (<code>ITask</code> instances) from specified 
 * XML-file to given <code>IRepositopyManager</code>
 * @author Eugenia
 *
 */
public class TasksReader implements TasksIOConstants {
    private final String myFileName;

    /**
     * Creates <code>TasksReader</code> for reading tasks from specified 
     * XML-file.
     * @param fileName name of the given XML-file to read tasks from
     */
    public TasksReader(String fileName) {
        myFileName = fileName;
    }

    /**
     * Reads tasks (<code>ITask</code> instances) from XML-file specified 
     * in constructor to given <code>IRepositopyManager</code>
     * @param repositopyManager the repositopyManager to read teasks into
     * @throws LoadException if there are any problems in reading data 
     * (some other Exceptions in loading mechanism occurs)
     */
    public void read(IRepositopyManager repositopyManager) throws LoadException {        
        try {            
            DocumentBuilderFactory docFactory = DocumentBuilderFactory
                .newInstance();            
            docFactory.setIgnoringComments(true);
            docFactory.setIgnoringElementContentWhitespace(true);
            docFactory.setValidating(false);
            DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
            Document doc = docBuilder.parse(new File(myFileName));
            Element rootElement = doc.getDocumentElement();
            NodeList tasksList = rootElement.getChildNodes();
            for (int i = 0; i < tasksList.getLength(); i++) {
                Node taskNode = tasksList.item(i);
                if (taskNode instanceof Element) {
                    Element taskElement = (Element) taskNode;
                    ITask task = readTask(taskElement, repositopyManager);
                    repositopyManager.add(task);
                }
            }

        } catch( Exception e ) {
            throw new LoadException(e.getMessage());
        }
    }

    /**
     * Parses <code>ITask</code> object from the given <code>Element</code>
     * using given <code>IRepositopyManager</code>
     * @param taskElement to parse <code>ITask</code> object from
     * @param repositopyManager to create <code>ITask</code> object
     * @return <code>ITask</code> result of parsing
     * @throws DOMException
     * @throws ParseException
     */
    private ITask readTask(Element taskElement, IRepositopyManager repositopyManager) throws ParseException {
        Date date = DateUtil.getToday();
        ITask task = repositopyManager.createDayTask(date, date, "");
        NamedNodeMap taskAttributesList = taskElement.getAttributes();
        if (taskAttributesList != null) {
            for (int j = 0; j < taskAttributesList.getLength(); j++) {
                Attr attributeNode = (Attr) taskAttributesList.item(j);
                String tagName = attributeNode.getName();
                String textContent = attributeNode.getValue();
                if (IDENTITY_TAG_NAME.equals(tagName)) {
                    if (textContent.equals(Identity.DAY_TASK.toString())) {
                        task.setIdentity(Identity.DAY_TASK);
                    } else {
                        task.setIdentity(Identity.TIME_TASK);
                    }
                } else if (PRIORITY_TAG_NAME.equals(tagName)) {
                    if (textContent.equals(Priority.IMPORTANT.toString())) {
                        task.setPriority(Priority.IMPORTANT);
                    } else {
                        task.setPriority(Priority.NORMAL);
                    }
                } else if (TIME_LIMITATION_TAG_NAME.equals(tagName)) {
                    if (textContent.equals(TimeLimitation.LIMITED.toString())) {
                        task.setTimeLimitation(TimeLimitation.LIMITED);
                    } else {
                        task.setTimeLimitation(TimeLimitation.PERMANENT);
                    }
                } else if (EXECUTION_STATE_TAG_NAME.equals(tagName)) {
                    if (textContent
                        .equals(ExecutionState.IN_PROCESS.toString())) {
                        task.setExecutionState(ExecutionState.IN_PROCESS);
                    } else if (textContent.equals(ExecutionState.CANCELED
                        .toString())) {
                        task.setExecutionState(ExecutionState.CANCELED);
                    } else {
                        task.setExecutionState(ExecutionState.DONE);
                    }
                }
            }
        }
        NodeList taskNodesList = taskElement.getChildNodes();
        Date beginDate = task.getBeginDate();
        Date endDate = task.getEndDate();
        for (int j = 0; j < taskNodesList.getLength(); j++) {
            Node node = taskNodesList.item(j);
            if (node instanceof Element) {
                Element nodeElement = (Element) node;
                String tagName = nodeElement.getTagName();
                Text textNode = (Text) nodeElement.getFirstChild();
                String text = textNode.getData().trim();
                if (TEXT_TAG_NAME.equals(tagName)) {
                    task.setText(text);
                } else if (BEGIN_DATE_TAG_NAME.equals(tagName)) {
                    beginDate = DateUtil.parseFromXMLDateTime(text);
                } else if (END_DATE_TAG_NAME.equals(tagName)) {
                    endDate = DateUtil.parseFromXMLDateTime(text);
                }
            }
        }
        task.setDates(beginDate, endDate);
        return task;
    }

}
