/**
 * Contains classes for reading and writing objects.
 */
package ru.amse.ksenofontova.jina.io;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.util.Collection;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.model.ExecutionState;
import ru.amse.ksenofontova.jina.model.IRepositopyManager;
import ru.amse.ksenofontova.jina.model.ITask;
import ru.amse.ksenofontova.jina.model.Identity;
import ru.amse.ksenofontova.jina.model.Priority;
import ru.amse.ksenofontova.jina.model.TimeLimitation;

/**
 * Class for writing tasks (<code>ITask</code> instances) to specified 
 * XML-file from given <code>IRepositopyManager</code>
 * @author Eugenia
 *
 */
public class TasksWriter implements TasksIOConstants {
    
    private final String myFileName;
    
    /**
     * Creates <code>TasksWriter</code> for writing tasks to specified 
     * XML-file.
     * @param fileName name of the given XML-file to write tasks to
     */
    public TasksWriter (String fileName) {
        myFileName = fileName;
    }
    
    /**
     * Writes tasks (<code>ITask</code> instances) to XML-file specified 
     * in constructor from given <code>IRepositopyManager</code>
     * @param repositopyManager the repositopyManager to write teasks from
     * @throws SaveException when there are some problems with writing data
     * (not enought memory on disc)
     */
    public void write(IRepositopyManager repositopyManager) throws SaveException {
        Collection<ITask> myTasks = repositopyManager.getInRange(null, null);
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory
                .newInstance();
            DocumentBuilder docBuilder = factory.newDocumentBuilder();
            Document doc = docBuilder.newDocument();
            Element rootElement = doc.createElement(TASK_LIST_TAG_NAME);
            for (ITask task: myTasks) {
                Element taskElement = doc.createElement(TASK_TAG_NAME);
                
                Identity identity = task.getIdentity();
                if(identity != ITask.DEFAULT_IDENTITY) {                    
                    taskElement.setAttribute(IDENTITY_TAG_NAME
                        , task.getIdentity().toString());
                }

                Element textElement = doc.createElement(TEXT_TAG_NAME);
                textElement.setTextContent(task.getText());
                taskElement.appendChild(textElement);

                Priority priority = task.getPriority();
                if(priority != ITask.DEFAULT_PROIRITY) {
                    taskElement.setAttribute(PRIORITY_TAG_NAME,
                        task.getPriority().toString()); 
                }
                
                TimeLimitation timeLimitation = task.getTimeLimitation();
                if(timeLimitation != ITask.DEFAULT_TIME_LIMITATION) {
                    taskElement.setAttribute(TIME_LIMITATION_TAG_NAME
                        , task.getTimeLimitation().toString()); 
                }                

                Element beginDateElement = doc.createElement(BEGIN_DATE_TAG_NAME);
                String beginDateStr = DateUtil.formatToXMLDateTime(task
                    .getBeginDate());
                beginDateElement.setTextContent(beginDateStr);
                taskElement.appendChild(beginDateElement);

                Element endDateElement = doc.createElement(END_DATE_TAG_NAME);
                String endDateStr = DateUtil.formatToXMLDateTime(task
                    .getEndDate());
                endDateElement.setTextContent(endDateStr);
                taskElement.appendChild(endDateElement);

                ExecutionState executionState = task.getExecutionState();
                if(executionState != ITask.DEFAULT_EXECUTION_STATE) {
                    taskElement.setAttribute(EXECUTION_STATE_TAG_NAME
                        , task.getExecutionState().toString()); 
                }
                
                rootElement.appendChild(taskElement);
            }
            doc.appendChild(rootElement);
            TransformerFactory tFactory = TransformerFactory.newInstance();
            tFactory.setAttribute("indent-number", new Integer(4));
            Transformer transformer = tFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.transform(new DOMSource(doc), new StreamResult(new OutputStreamWriter(
                new FileOutputStream(new File(myFileName)))));
        } catch( Throwable t ) {
            throw new SaveException(t.getMessage());
        }
        
    }

}
