/**
 * Contains classes for reading and writing objects.
 */
package ru.amse.ksenofontova.jina.io;

import java.awt.Color;
import java.awt.Font;
import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;


import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.view.ViewParameters;

/**
 * Class for reading <code>ViewParameters'</code> fields from specified XML-file 
 * @author Eugenia
 */
public class ViewParametersReader implements ViewParametersIOConstants {
    private final String myFileName;
    
    /**
     * Creates <code>ViewParametersReader</code> for reading 
     * <code>ViewParameters'</code> fields from specified XML-file.
     * @param fileName
     *        name of the given XML-file to read <code>ViewParameters'</code>
     *        fields from
     * 
     */
    public ViewParametersReader (String fileName) {
        myFileName = fileName;
    }
    
    /**
     * Reads and sets <code>ViewParameters'</code> fields from XML-file 
     * specified in constructor
     * @throws LoadException if there are any problems in reading data 
     * (some other Exceptions in loading mechanism occurs)
     */
    public void read() throws LoadException {
        try {
          DocumentBuilderFactory docFactory = DocumentBuilderFactory
              .newInstance();
          docFactory.setIgnoringElementContentWhitespace(true);
          DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
          Document doc = docBuilder.parse(new File(myFileName));
          Element rootElement = doc.getDocumentElement();
          NodeList properties = rootElement.getChildNodes();
          for (int i = 0; i < properties.getLength(); i++) {
              Node propertyNode = properties.item(i);
              if (propertyNode instanceof Element) { 
                  Element propertyElement = (Element) propertyNode;                  
                  String tagName = propertyElement.getTagName();
                  
                  if (SCALE_TAG_NAME.equals(tagName)) {
                      Text textNode = (Text)propertyNode.getFirstChild();
                      ViewParameters.setScale(Integer.parseInt
                          (textNode.getData().trim()));
                  } else if (BORDER_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.BorderColor = parseColor(propertyElement);
                  } else if (FIRST_VISIBLE_DATE_TAG_NAME.equals(tagName)) {
                	  Text textNode = (Text)propertyNode.getFirstChild();
                	  String text = textNode.getData().trim();
                      ViewParameters.setFirstVisibleDate(DateUtil.parseFromXMLDateTime(text));
                  } else if (GRID_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.GridColor = parseColor(propertyElement);
                  } else if (TODAY_GRID_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.TodayGridColor = parseColor(propertyElement);
                  } else if (VIEW_FONT_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.ViewFontColor = parseColor(propertyElement);
                  } else if (WEEKEND_FONT_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.WeekendFontColor = parseColor(propertyElement);
                  } else if (VIEW_BACKGROUND_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.ViewBackgroundColor = parseColor(propertyElement);
                  } else if (WORK_DAY_TASKS_PANEL_BACKGROUND_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.WorkDayTasksPanelBackgroundColor = parseColor(propertyElement);
                  } else if (WEEKEND_TASKS_PANEL_BACKGROUND_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.WeekendTasksPanelBackgroundColor = parseColor(propertyElement);
                  } else if (PAST_TASKS_PANEL_BACKGROUND_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.PastTasksPanelBackgroundColor = parseColor(propertyElement);
                  } else if (TASK_BORDER_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.TaskBorderColor = parseColor(propertyElement);
                  } else if (SELECTED_TASK_BACKGROUND_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.SelectedTaskBackgroundColor = parseColor(propertyElement);
                  } else if (SELECTED_TASK_FONT_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.SelectedTaskFontColor = parseColor(propertyElement);
                  } else if (TASK_BACKGROUND_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.TaskBackgroundColor = parseColor(propertyElement);
                  } else if (TASK_FONT_COLOR_TAG_NAME.equals(tagName)) {
                      ViewParameters.TaskFontColor = parseColor(propertyElement);
                  } else if (TASK_FONT_TAG_NAME.equals(tagName)) {
                      ViewParameters.setTaskFont(parseFont(propertyElement));
                  }
              }
          }

      } catch( Throwable e ) {
          throw new LoadException(e.getMessage());
      }
    }

    /**
     * Parses <code>Font</code> object from the given <code>Element</code> 
     * @param propertyElement the Element to parse <code>Font</code> from
     * @return parsed <code>Font</code> object
     */
    private Font parseFont(Element propertyElement) {
        String name = propertyElement.getAttribute(FONT_NAME_TAG_NAME);
        int style = Integer.parseInt(propertyElement.getAttribute(FONT_STYLE_TAG_NAME));
        int size = Integer.parseInt(propertyElement.getAttribute(FONT_SIZE_TAG_NAME));
        return new Font(name, style, size);
    }
    

    /**
     * Parses <code>Color</code> object from the given <code>Element</code>
     * @param propertyElement the Element to parse <code>Color</code> from
     * @return parsed <code>Color</code> object
     */
    private Color parseColor(Element propertyElement) {
        int red = Integer.parseInt(propertyElement.getAttribute(RED_TAG_NAME));
        int green = Integer.parseInt(propertyElement.getAttribute(GREEN_TAG_NAME));
        int blue = Integer.parseInt(propertyElement.getAttribute(BLUE_TAG_NAME));
        int alpha = Integer.parseInt(propertyElement.getAttribute(ALPHA_TAG_NAME));
        return new Color(red, green, blue, alpha);
    }

}
