/**
 * Contains classes for reading and writing objects.
 */
package ru.amse.ksenofontova.jina.io;

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.view.ViewParameters;

/**
 * Class for writing <code>ViewParameters'</code> fields to specified XML-file
 * @author Eugenia
 *
 */
public class ViewParametersWriter implements ViewParametersIOConstants {
    private final String myFileName;
    
    /**
     * Creates <code>ViewParametersWriter</code> for writing 
     * <code>ViewParameters'</code> fields to specified XML-file.
     * @param fileName name of the given XML-file to write 
     * <code>ViewParameters'</code> fields
     */
    public ViewParametersWriter (String fileName) {
        myFileName = fileName;
    }
    
    /**
     * Writes <code>ViewParameters'</code> fields to XML-file specified 
     * in constructor
     * @throws SaveException when there are some problems with writing data
     * (not enought memory on disc)
     */
    public void write() throws SaveException {
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory
                .newInstance();
            DocumentBuilder docBuilder = factory.newDocumentBuilder();
            Document doc = docBuilder.newDocument();
            Element rootElement = doc.createElement(VIEW_PARAMETERS_TAG_NAME);
            
            if(ViewParameters.getScale() != ViewParameters.DEFAULT_SCALE) {                
                Element scaleElement = doc.createElement(SCALE_TAG_NAME);
                scaleElement.setTextContent("" + ViewParameters.getScale());
                rootElement.appendChild(scaleElement);
            }
            
            Element firstVisibleDateElement = doc.createElement(FIRST_VISIBLE_DATE_TAG_NAME);
            String firstVisibleDateStr = DateUtil.formatToXMLDateTime(ViewParameters.getFirstVisibleDate());
            firstVisibleDateElement.setTextContent(firstVisibleDateStr);
            rootElement.appendChild(firstVisibleDateElement);
            
            if(ViewParameters.BorderColor != ViewParameters.DEFAULT_BORDER_COLOR) {
                Element colorElement = doc.createElement(BORDER_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.BorderColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.GridColor != ViewParameters.DEFAULT_GRID_COLOR) {
                Element colorElement = doc.createElement(GRID_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.GridColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.TodayGridColor != ViewParameters.DEFAULT_TODAY_GRID_COLOR) {
                Element colorElement = doc.createElement(TODAY_GRID_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.TodayGridColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.ViewFontColor != ViewParameters.DEFAULT_VIEW_FONT_COLOR) {
                Element colorElement = doc.createElement(VIEW_FONT_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.ViewFontColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.WeekendFontColor != ViewParameters.DEFAULT_WEEKEND_FONT_COLOR) {
                Element colorElement = doc.createElement(WEEKEND_FONT_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.WeekendFontColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.ViewBackgroundColor != ViewParameters.DEFAULT_VIEW_BACKGROUND_COLOR) {
                Element colorElement = doc.createElement(VIEW_BACKGROUND_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.ViewBackgroundColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.WorkDayTasksPanelBackgroundColor != ViewParameters.DEFAULT_WORK_DAY_TASKS_PANEL_BACKGROUND_COLOR) {
                Element colorElement = doc.createElement(WORK_DAY_TASKS_PANEL_BACKGROUND_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.WorkDayTasksPanelBackgroundColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.WeekendTasksPanelBackgroundColor != ViewParameters.DEFAULT_WEEKEND_TASKS_PANEL_BACKGROUND_COLOR) {
                Element colorElement = doc.createElement(WEEKEND_TASKS_PANEL_BACKGROUND_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.WeekendTasksPanelBackgroundColor);
                rootElement.appendChild(colorElement);
            }            
            
            if(ViewParameters.PastTasksPanelBackgroundColor 
                != ViewParameters.DEFAULT_PAST_TASKS_PANEL_BACKGROUND_COLOR) {
                Element colorElement = doc.createElement(PAST_TASKS_PANEL_BACKGROUND_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.PastTasksPanelBackgroundColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.TaskBorderColor != ViewParameters.DEFAULT_TASK_BORDER_COLOR) {
                Element colorElement = doc.createElement(TASK_BORDER_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.TaskBorderColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.SelectedTaskBackgroundColor != ViewParameters.DEFAULT_SELECTED_TASK_BACKGROUND_COLOR) {
                Element colorElement = doc.createElement(SELECTED_TASK_BACKGROUND_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.SelectedTaskBackgroundColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.SelectedTaskFontColor != ViewParameters.DEFAULT_SELECTED_TASK_FONT_COLOR) {
                Element colorElement = doc.createElement(SELECTED_TASK_FONT_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.SelectedTaskFontColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.TaskBackgroundColor != ViewParameters.DEFAULT_TASK_BACKGROUND_COLOR) {
                Element colorElement = doc.createElement(TASK_BACKGROUND_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.TaskBackgroundColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.TaskFontColor != ViewParameters.DEFAULT_TASK_FONT_COLOR) {
                Element colorElement = doc.createElement(TASK_FONT_COLOR_TAG_NAME);
                writeColor(colorElement, ViewParameters.TaskFontColor);
                rootElement.appendChild(colorElement);
            }
            
            if(ViewParameters.getTaskFont() != ViewParameters.DEFAULT_TASK_FONT) {
                Element fontElement = doc.createElement(TASK_FONT_TAG_NAME);
                writeFont(fontElement, ViewParameters.getTaskFont());
                rootElement.appendChild(fontElement);
            }
            
            doc.appendChild(rootElement);
            TransformerFactory tFactory = TransformerFactory.newInstance();
            tFactory.setAttribute("indent-number", new Integer(4));
            Transformer transformer = tFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.transform(new DOMSource(doc), new StreamResult(new OutputStreamWriter(
                new FileOutputStream(new File(myFileName)))));
        } catch( Throwable t ) {
            throw new SaveException(t.getMessage());
        }
        
    }

    /**
     * Writes given <code>Font</code> object to the given <code>Element</code>
     * @param fontElement the <code>Element</code> to write Font to
     * @param taskFont the given Font to write
     */
    private void writeFont(Element fontElement, Font taskFont) {
        fontElement.setAttribute(FONT_SIZE_TAG_NAME, taskFont.getSize() + "");
        fontElement.setAttribute(FONT_STYLE_TAG_NAME, taskFont.getStyle() + "");
        fontElement.setAttribute(FONT_NAME_TAG_NAME, taskFont.getName() + "");
                
    }
    
    

    /**
     * Writes given <code>Color</code> object to the given <code>Element</code>
     * @param colorElement the <code>Element</code> to write Color to
     * @param viewFontColor the given Color to write
     */
    private void writeColor(Element colorElement, Color color) {
        colorElement.setAttribute(BLUE_TAG_NAME, color.getBlue() + "");
        colorElement.setAttribute(GREEN_TAG_NAME, color.getGreen() + "");        
        colorElement.setAttribute(RED_TAG_NAME, color.getRed() + "");
        colorElement.setAttribute(ALPHA_TAG_NAME, color.getAlpha() + "");
    }

}
