/**
 * Contains representations of tasks which user can plan.
 */
package ru.amse.ksenofontova.jina.model;

import java.text.DateFormat;
import java.text.DateFormatSymbols;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import ru.amse.ksenofontova.jina.MainParameters;

/**
 * Utility class for operation with <code>Date</code> objects. 
 * First of all, this class provides some specific comparison methods 
 * (<code>getDaysDiff(date1, date2), getHoursDiff(date1, date2)).
 * <p><code>getDaysDiff(date1, date2)</code> calculates difference in days 
 * between two given dates without taking into consideration their time 
 * fields (hours, minutes etc).
 * <p><code>getHoursDiff(date1, date2)</code> calculates difference between 
 * two given dates in hours without taking into consideration "low" time 
 * fields (minutes, secunds etc).
 * <p>It is intended to simplify creating <code>Date's</code> objects with 
 * specified parameters such as year, month, day; getting and setting 
 * <code>Date's</code> fields by incapsulating all the work with 
 * <code>Calendar</code>. 
 * It simplifies some frequently used operationst of formatting and parsing
 * date strings  by incapsulating all the work with <code>DateFormat</code>.
 * All above-listed operations implementation is based on two implicit
 * parameters spesified in <code>MainParameters</code> 
 * (MainParameters.CurrentTimeZone, MainParameters.CurrentLocale).
 * @see Date
 * <p>Calendar, GregorianCalendar
 * <p>MainParameters
 * @author Eugenia
 * 
 */
public class DateUtil { 
    
    private DateUtil(){}
   
    private static final GregorianCalendar ourCalendar 
        = new GregorianCalendar(MainParameters.CurrentTimeZone
            , MainParameters.CurrentLocale);
    
    private static final DateFormatSymbols ourDateFormatSymbols 
        = new DateFormatSymbols(MainParameters.CurrentLocale);
    
    /**
     * To forbid creating date objects with illegal attributes.
     */
    static {
        ourCalendar.setLenient(false);
    }
    
          
    
    /**
     * Returnes count of days in spesified month in spesified year
     * @param month (0 based)
     * @param year (after 1 January 1970 0:00 UTC)
     * @return maximum days in spesified month in spesified year
     */
    public static final int getDaysInMonth(Date currentMonthDate){
        ourCalendar.setTime(currentMonthDate);
        ourCalendar.add(Calendar.MONTH, 1);
        ourCalendar.set(Calendar.DATE, 1);
        ourCalendar.add(Calendar.DATE, -1);
        return ourCalendar.get(Calendar.DATE);        
    }
         
    
    /**
     * Determines if day of week represented by the given <code>Date</code>
     * object is weekend (<code>SUNDAY</code> or <code>SATURDAY</code>). 
     * @param date
     * @return true if day of week represented by the given 
     * <code>Date</code> object is <code>SUNDAY</code> or 
     * <code>SATURDAY</code>, false otherwise.
     */
    public static boolean isWeekend(Date date) {
        ourCalendar.setTime(date);        
        int dayOfWeek = ourCalendar.get(Calendar.DAY_OF_WEEK);
        return (dayOfWeek == Calendar.SUNDAY)
              ||(dayOfWeek == Calendar.SATURDAY);
    }
    
    /**
     * Creates <code>Date</code> object by calling following constructor
     * <p><code>new Date(System.currentTimeMillis());</code>
     * @return <code>Date</code> object represents current day.
     */
    public static Date getToday() {
        return new Date(System.currentTimeMillis());
    }
    
    /**
     * Creates <code>Date</code> object with specified year, month and day, 
     * calls <code>clear()</code> method of the internal 
     * <code>Calendar</code> first.
     * @param year the value used to set the <code>YEAR</code> calendar field.
     * @param month the value used to set the <code>MONTH</code> calendar field.
     * Month value is 0-based. e.g., 0 for January.
     * @param date the value used to set the <code>DAY_OF_MONTH</code> calendar field.
     * @throws IllegalArgumentException if some of parameters is out of range
     * @see Calendar.set(int,int,int,int,int)
     * @see #newDate(int,int,int,int,int)
     * @return <code>Date</code> object with specified year, month and day.
     */
    public static Date newDate(int year, int month, int day){
        ourCalendar.clear();
        ourCalendar.set(year, month, day);
        return ourCalendar.getTime();
    }    
    
    
    /**
     * Creates <code>Date</code> object with specified year, month, day, 
     * hourOfDay and minute fields. Calls <code>clear()</code> method of 
     * the internal <code>Calendar</code> first.
     * @param year the value used to set the <code>YEAR</code> calendar field.
     * @param month the value used to set the <code>MONTH</code> calendar field.
     * Month value is 0-based. e.g., 0 for January.
     * @param date the value used to set the <code>DAY_OF_MONTH</code> calendar field.
     * @param hourOfDay the value used to set the <code>HOUR_OF_DAY</code> calendar field.
     * @param minute the value used to set the <code>MINUTE</code> calendar field.     * 
     * @see Calendar.set(int,int,int,int,int)
     * @see #newDate(int,int,int)
     * @return <code>Date</code> object with specified year, month, day, 
     * hourOfDay and minute fields.
     */
    public static Date newDate(int year, int month, int day
        , int hourOfDay, int minute){
        ourCalendar.clear();
        ourCalendar.set(year, month, day, hourOfDay, minute);
        return ourCalendar.getTime();
        
    }  
    
    /**
     * Calculates difference in days between two given dates (date1 - date2) 
     * without taking into consideration their time fields (hours, minutes etc).
     * Takes into consideration <code>Calendar.DST_OFFSET</code> field.
     * Assumes count of milliseconds in one day as 24*60*60*1000.
     * @param date1 <code>Date</code> object that represents first date
     * @param date2 <code>Date</code> object that represents second date
     * @see Calendar.DST_OFFSET
     * @return difference in days between two given dates
     */
    public static int getDaysDiff(Date date1, Date date2){
        ourCalendar.setTime(date1);
        ourCalendar.set(Calendar.HOUR_OF_DAY, 0);
        ourCalendar.set(Calendar.MINUTE, 0);
        ourCalendar.set(Calendar.SECOND, 0);
        ourCalendar.set(Calendar.MILLISECOND, 0);
        int dstOffset1 = ourCalendar.get(Calendar.DST_OFFSET);
        long date1InMillis = ourCalendar.getTimeInMillis() + dstOffset1;
        
        ourCalendar.setTime(date2);
        ourCalendar.set(Calendar.HOUR_OF_DAY, 0);
        ourCalendar.set(Calendar.MINUTE, 0);
        ourCalendar.set(Calendar.SECOND, 0);
        ourCalendar.set(Calendar.MILLISECOND, 0);
        int dstOffset2 = ourCalendar.get(Calendar.DST_OFFSET);
        long date2InMillis = ourCalendar.getTimeInMillis() + dstOffset2;
        
        long diffInMillis = date1InMillis - date2InMillis;
        // TODO : remove magic numbers???
        long dayInMillis = 24*60*60*1000;
        long result = diffInMillis/dayInMillis;               
        return (int)result;
    }
    
    /**
     * Calculates difference in hours between two given dates (date1 - date2) 
     * without taking into consideration their "low" time fields 
     * (minutes, secunds etc).
     * Takes into consideration <code>Calendar.DST_OFFSET</code> field.
     * Assumes count of milliseconds in one hour as 60*60*1000.
     * @param date1 <code>Date</code> object that represents first date
     * @param date2 <code>Date</code> object that represents second date
     * @see Calendar.DST_OFFSET
     * @return difference in days between two given dates
     */
    public static int getHoursDiff(Date date1, Date date2){
        ourCalendar.setTime(date1);
        ourCalendar.set(Calendar.SECOND, 0);
        ourCalendar.set(Calendar.MILLISECOND, 0);
        int dstOffset1 = ourCalendar.get(Calendar.DST_OFFSET);
        long date1InMillis = ourCalendar.getTimeInMillis() + dstOffset1;
        
        ourCalendar.setTime(date2);
        ourCalendar.set(Calendar.SECOND, 0);
        ourCalendar.set(Calendar.MILLISECOND, 0);
        int dstOffset2 = ourCalendar.get(Calendar.DST_OFFSET);
        long date2InMillis = ourCalendar.getTimeInMillis() + dstOffset2;
        
        long diffInMillis = date1InMillis - date2InMillis;
        // TODO : remove magic numbers???
        long hourInMillis = 60*60*1000;
        long result = diffInMillis/hourInMillis;               
        return (int)result;
    }
    
    
    /**
     * Sets the specified values for the <code>HOUR_OF_DAY</code>,
     * and <code>MINUTE</code> of the given Date. For <code>MILLISECOND</code>
     * sets value 0. Previous values of other fields are retained. 
     * @param date the given Date
     * @param hourOfDay to set <code>HOUR_OF_DAY</code> calendar field 
     * that used for the 24-hour clock.
     * E.g., at 10:04:15.250 PM the <code>HOUR_OF_DAY</code> is 22.
     * @param minute the value used to set the <code>MINUTE</code> calendar field.
     * @return Date with specified values of its fields.
     */
    public static Date setTime(Date date, int hourOfDay, int minute){
        ourCalendar.setTime(date);
        ourCalendar.set(Calendar.HOUR_OF_DAY, hourOfDay);
        ourCalendar.set(Calendar.MINUTE, minute);
        return ourCalendar.getTime();
    }
    
    
    /**
     * Adds the specified (signed) count of daysto the given Date, based on
     * the calendar's rules.
     *
     * <p> The value of <code>day</code>
     * after the call minus the value of <code>day</code> before the
     * call is <code>count</code>, modulo any overflow that has occurred in
     * <code>day</code>. Overflow occurs when a day value exceeds its
     * range and, as a result, the next field year is incremented or
     * decremented and the field value is adjusted back into its range.</p> 
     * 
     * @param count the count of days to be added to the given Date 
     * (<code>Calendar.DATE</code> field).
     * @param date the given Date
     * @return Date after adding days.
     * @see     java.util.Calendar 
     * <p><code>Calendar.add(Calendar.DATE, count)</code>.
     * @exception IllegalArgumentException if any calendar fields have
     * out-of-range values in non-lenient mode.
     */
    public static Date addDays(Date date, int count){        
        ourCalendar.setTime(date);
        ourCalendar.add(Calendar.DATE, count);
        return ourCalendar.getTime();
    }
    

    /**
     * Returns the value of the <code>Calendar.YEAR</code> field.
     * @param date the given Date
     * @return int value of the <code>Calendar.YEAR</code> field
     * @see     java.util.Calendar 
     * <p><code>Calendar.get(Calendar.YEAR)</code>.
     */
    public static int getYear(Date date){
        ourCalendar.setTime(date);
        return ourCalendar.get(Calendar.YEAR);
    }
        
    
    /**
     * Returns the value of the <code>Calendar.MONTH</code> field of
     * the given date wich is in [0, 11] range
     * @param date the given Date
     * @return int value of the <code>Calendar.MONTH</code> field in [0;11] range
     * @see     java.util.Calendar 
     * <p><code>Calendar.get(Calendar.MONTH)</code>.
     */
    public static int getMonth(Date date){
        ourCalendar.setTime(date);
        return ourCalendar.get(Calendar.MONTH);
    }
        
    
    /**
     * Returns the value of the <code>Calendar.DAY_OF_MONTH</code> field of
     * the given date wich is in [1, 31] range (depends on month).
     * @return int value of the <code>Calendar.DAY_OF_MONTH</code> field 
     * in [1, 31] range
     * @param date the given Date
     * @see     java.util.Calendar 
     * <p><code>Calendar.get(Calendar.DAY_OF_MONTH)</code>.
     */
    public static int getDay(Date date){
        ourCalendar.setTime(date);
        return ourCalendar.get(Calendar.DAY_OF_MONTH);
    }
    
    
    /**
     * Returns the day of the week represented by given <code>date</code>.
     * The returned value (<tt>1</tt> = Sunday, <tt>2</tt> = Monday, 
     * <tt>3</tt> = Tuesday, <tt>4</tt> = Wednesday, <tt>5</tt> = 
     * Thursday, <tt>6</tt> = Friday, <tt>7</tt> = Saturday) 
     * represents the day of the week that contains or begins with 
     * the instant in time represented by given <tt>Date</tt> object, 
     * as interpreted in the ViewParameters.CurrentTimeZone.
     *
     * @param date the given Date
     * @return the day of the week represented by given date.
     * @see     java.util.Calendar 
     * <p><code>Calendar.get(Calendar.DAY_OF_WEEK)</code>.
     */ 
    public static int getDayOfWeek(Date date){
        ourCalendar.setTime(date);
        return ourCalendar.get(Calendar.DAY_OF_WEEK);
    }
    

    /**
     * Returns string representstion of the  day of the week represented by
     * the given <code>date</code>.
     *
     * @param date the given Date
     * @return string representstion of the day of week represented by given date.
     * @see ru.amse.ksenofontova.jina.model.DateUtil 
     * <p><code>DateUtil.getDayOfWeek(Date date)</code>.
     * @see java.util.Calendar 
     * <p><code>Calendar.get(Calendar.DAY_OF_WEEK)</code>.
     */
    public static String getDayOfWeekString(Date date){
        ourCalendar.setTime(date);
        return (ourDateFormatSymbols.getWeekdays())
            [ourCalendar.get(Calendar.DAY_OF_WEEK)];
    }
    

    /**
     * Returns string representstion of the month represented by
     * the given <code>date</code>.
     *
     * @param date the given Date
     * @return string representstion of the month represented by given date.
     * @see java.util.Calendar 
     * <p><code>Calendar.getMonths()</code>.
     */
    public static String getMonthString(Date date){
        ourCalendar.setTime(date);
        return (ourDateFormatSymbols.getMonths())
            [ourCalendar.get(Calendar.MONTH)];
    }


    /**
     * Returns short string representstion of the month represented by
     * the given <code>Date</code>.
     *
     * @param date the given Date
     * @return string representstion of the month represented by given date.
     */
    public static String getShortMonthString(Date date){
        ourCalendar.setTime(date);
        return (ourDateFormatSymbols.getShortMonths())
            [ourCalendar.get(Calendar.MONTH)];
    }

    
    /**
     * Returns short string representstion of the  day of the week represented by
     * the given <code>Date</code>.
     *
     * @param date the given Date
     * @return the day of the week represented by given date.
     * @see ru.amse.ksenofontova.jina.model.DateUtil 
     * <p><code>DateUtil.getDayOfWeek(Date date)</code>.
     * @see java.util.Calendar 
     * <p><code>Calendar.get(Calendar.DAY_OF_WEEK)</code>.
     */
    public static String getShortDayOfWeekString(Date date){
        ourCalendar.setTime(date);
        return (ourDateFormatSymbols
            .getShortWeekdays())[ourCalendar.get(Calendar.DAY_OF_WEEK)];
    }
    


    /**
     * Returns string representstion of the time represented by
     * the given <code>Date</code>.
     *
     * @param date the given Date
     * @return the the time represented by given date.
     */
    public static String getTimeString(Date date){
        DateFormat df = DateFormat.getTimeInstance(DateFormat.SHORT
            , MainParameters.CurrentLocale);
        return df.format(date);
    }   
    
    
    /**
     * Returns the value of the <code>Calendar.HOUR_OF_DAY</code> field of
     * the given date wich is in [0, 23] range.
     * @return int value of the <code>Calendar.HOUR_OF_DAY</code> field 
     * in [0, 23] range
     * @param date the given Date
     * @see     java.util.Calendar 
     * <p><code>Calendar.get(Calendar.HOUR_OF_DAY)</code>.
     */
    public static int getHour(Date date){
        ourCalendar.setTime(date);
        return ourCalendar.get(Calendar.HOUR_OF_DAY);
    }
        
    
    /**
     * Returns the value of the <code>Calendar.MINUTE</code> field of
     * the given date wich is in [0, 59] range.
     * @return int value of the <code>Calendar.MINUTE</code> field 
     * in [0, 59] range
     * @param date the given Date
     * @see     java.util.Calendar 
     * <p><code>Calendar.get(Calendar.MINUTE)</code>.
     */
    public static int getMinute(Date date){
        ourCalendar.setTime(date);
        return ourCalendar.get(Calendar.MINUTE);
    }
    
    /**
     * Formats given date into short date/time String in Locale specified
     * by MainParameters.CurrentLocale
     * @param date the given Date
     * @return string representation of the given Date formatted in the 
     * <code>DateFormat.SHORT</code> style in the 
     * <code>MainParameters.CurrentLocale</code> locale.
     * @see java.text.DateFormat
     */
    public static String getDateString(Date date) {
        DateFormat dateFormat = DateFormat.getDateInstance(DateFormat.SHORT,
            MainParameters.CurrentLocale);
        return dateFormat.format(date);
    }
    
    /**
     * Parses <code>Date</code> object from the <code>String</code> wich format
     * is specified by "ISO 8601 Date and Time Formats". This format used in
     * "xsd:dateTime" type in XML Schemas.
     * <p>According to this standart dateTime values may be viewed as objects
     * with integer-valued year, month, day, hour and minute properties, a 
     * decimal-valued second property, and a boolean timezoned property. 
     * Each such object also has one decimal-valued method or computed 
     * property, timeOnTimeline, whose value is always a decimal number; 
     * the values are dimensioned in seconds, the integer 0 is 
     * 0001-01-01T00:00:00 and the value of timeOnTimeline for other dateTime
     * values is computed using the Gregorian algorithm as modified for 
     * leap-seconds. The timeOnTimeline values form two related "timelines",
     * one for timezoned values and one for non-timezoned values. 
     * Each timeline is a copy of the value space of decimal, with integers
     * given units of seconds.
     * @param dateTimeStr the given String in formatted according to 
     * "ISO 8601 Date and Time Formats".
     * @throws ParseException if the given string cannot be parsed
     * @see java.text.NumberFormat
     */
    public static Date parseFromXMLDateTime(String dateTimeStr) throws ParseException {
        NumberFormat nf = NumberFormat.getIntegerInstance();
        ourCalendar.set(Calendar.YEAR
            , nf.parse(dateTimeStr.substring(0, 4)).intValue());
        ourCalendar.set(Calendar.MONTH
            , nf.parse(dateTimeStr.substring(5, 7)).intValue() - 1);
        ourCalendar.set(Calendar.DATE
            , nf.parse(dateTimeStr.substring(8, 10)).intValue());
        ourCalendar.set(Calendar.HOUR_OF_DAY
            , nf.parse(dateTimeStr.substring(11, 13)).intValue());
        ourCalendar.set(Calendar.MINUTE
            , nf.parse(dateTimeStr.substring(14, 16)).intValue());
        ourCalendar.set(Calendar.SECOND
            , nf.parse(dateTimeStr.substring(17, 19)).intValue());
        return ourCalendar.getTime();
    }
    
    /**
     * Formats given date into string according to the "ISO 8601 Date and 
     * Time Formats".
     * @param date the given date which is to be formatted
     * @return string formatted according to the "ISO 8601 Date and 
     * Time Formats".
     */
    public static String formatToXMLDateTime(Date date) {
        return String.format ("%1$tFT%1$tT", date);
    }
    
    
    
    /**
     * Tests if given <code>checkedDate</code> is in range 
     * [ <code>rangeBegin, rangeEnd</code> ]
     * @param checkedDate date to check
     * @param rangeBegin Date which represents the beginning of the range
     * @param rangeEnd Date which represents the ending of the range
     * @return <code>true</code> in four cases:
     * <p>case1: null <= checkedDate <= null
     * <p>case2: null <= checkedDate <= rangeEnd
     * <p>case3: rangeBegin <= checkedDate <= null
     * <p>case4: rangeBegin <= checkedDate <= rangeEnd
     * <p>Returnes <code>false</code> otherwise
     * @throws NullPointerException if checked Date is null 
     */
    public static boolean isInRange(Date checkedDate, Date rangeBegin
        , Date rangeEnd) {
        boolean result = false;
        if(rangeBegin == null){
           result = (rangeEnd == null)
               // null <= checkedDate <= null
               ? true 
               // null <= checkedDate <= rangeEnd
               : getDaysDiff(checkedDate, rangeEnd) <= 0; 
        } else {
            result = (rangeEnd == null)
                // rangeBegin <= checkedDate <= null
                ? getDaysDiff(rangeBegin, checkedDate) <= 0 
                // rangeBegin <= checkedDate <= rangeEnd
                : (getDaysDiff(rangeBegin, checkedDate) <= 0) 
                  && (getDaysDiff(checkedDate, rangeEnd) <= 0); 
        }
        return result;
    }
    
         
}
