/**
 * Contains representations of tasks which user can plan.
 */
package ru.amse.ksenofontova.jina.model;

import java.util.Date;

/**
 * <code>ITask</code> interface represents something that user can plan. It
 * has period (specified by two Dates) in which it is to be done, text 
 * description and some other attributes.
 * @author Eugenia
 *
 */
public interface ITask {
    /**
     * Defalt value of <code>Identity</code> task attribute. If no other
     * value is set for this field it will have this one.
     */
    Identity DEFAULT_IDENTITY = Identity.DAY_TASK;
    
    /**
     * Defalt value of <code>Priority</code> task attribute. If no other
     * value is set for this field it will have this one.
     */
    Priority DEFAULT_PROIRITY = Priority.NORMAL;
    
    /**
     * Defalt value of <code>ExecutionState</code> task attribute. If no other
     * value is set for this field it will have this one.
     */
    ExecutionState DEFAULT_EXECUTION_STATE = ExecutionState.IN_PROCESS;
    
    /**
     * Defalt value of <code>TimeLimitation</code> task attribute. If no other
     * value is set for this field it will have this one.
     */
    TimeLimitation DEFAULT_TIME_LIMITATION = TimeLimitation.LIMITED;    

    /**
     * Returns the date which represents the beginning of the period the task
     * has been planned for.
     * @return the date which represents the beginning of the period the task
     * has been planned for.
     */
    public abstract Date getBeginDate();

    
    /**
     * Returns the date which represents the completion of the period the task
     * has been planned for.
     * @return the date which represents the completion of the period the task
     * has been planned for.
     */
    public abstract Date getEndDate();

    //TODO:    !!!
    /**
     * Returns <code>Identity</code> task attribute specifies whether this
     * <code>ITask</code> instance is a task for some days or a task for 
     * some hours during one day.
     * @return <code>Identity</code> task attribute specifies whether this
     * <code>ITask</code> instance is a task for some days or a task for 
     * some hours during one day.
     * @see ru.amse.ksenofontova.jina.model.Identity
     */
    public abstract Identity getIdentity();

    /**
     * Returns <code>Priority</code> task attribute.
     * @return <code>Priority</code> task attribute.
     * @see ru.amse.ksenofontova.jina.model.Priority
     */
    public abstract Priority getPriority();

    /**
     * Returns <code>ExecutionState</code> task attribute.
     * @return <code>ExecutionState</code> task attribute.
     * @see ru.amse.ksenofontova.jina.model.ExecutionState
     */
    public abstract ExecutionState getExecutionState();

    /**
     * Returns task's text description.
     * @return String object represents task's text description.
     */
    public abstract String getText();

    /**
     * Tells whether this task has <code>TimeLimitation</code> field set at
     *  {@link TimeLimitation#PERMANENT} value or not.
     * @return true if this task has <code>TimeLimitation</code> field set at
     *  {@link TimeLimitation#PERMANENT} value, false otherwise.
     */
    public boolean isPermanent();
    
    /**
     * Returns <code>TimeLimitation</code> task attribute.
     * @return <code>TimeLimitation</code> task attribute.
     * @see ru.amse.ksenofontova.jina.model.TimeLimitation
     */
    public abstract TimeLimitation getTimeLimitation();    

    
    /**
     * Sets values for both beginDate and endDate task's fields.
     * @param beginDate the date which represents the beginning of the period the task
     * has been planned for.
     * @param endDate the date which represents the completion of the period the task
     * has been planned for.
     * @throws WrongTimeLimitException if endDate <= beginDate
     */
    public void setDates(Date beginDate, Date endDate) throws IllegalArgumentException;
    
    /**
     * Sets specified value for the <code>Identity</code> task attribute.
     * @param identity value which is to be set for the <code>Identity</code>
     * task attribute.
     */
    public abstract void setIdentity(Identity identity);

    /**
     * Sets specified value for the <code>Priority</code> task attribute.
     * @param identity value which is to be set for the <code>Priority</code>
     * task attribute.
     */
    public abstract void setPriority(Priority priority);

    /* TODO
     *      setDoneState(); 
     * setCanceledState(),    
     *     (PERMANENT&DONE; PERMANENT&CANCELED)
     */
    /**
     * Sets specified value for the <code>ExecutionState</code> task attribute.
     * @param identity value which is to be set for the <code>ExecutionState</code>
     * task attribute.
     */
    public abstract void setExecutionState(ExecutionState state);

    /**
     * Sets task's text description.
     * @param text for task description.
     */
    public abstract void setText(String text);

    /**
     * Sets specified value for the <code>TimeLimitation</code> task attribute.
     * @param identity value which is to be set for the <code>TimeLimitation</code>
     * task attribute.
     */
    public abstract void setTimeLimitation(TimeLimitation timeLimitation);

    /**
     * Compares two objects for equality.
     * The result is <code>true</code> if and only if the argument is 
     * not <code>null</code> and is a instance of <code>ITask</code>
     * with equal attributes, as this object.
     * 
     * @param   obj   the object to compare with.
     * @return <code>true</code> if and only if the argument is 
     * not <code>null</code> and is a instance of <code>ITask</code>
     * with equal attributes, as this object;
     *         <code>false</code> otherwise.
     */
    public abstract boolean equals(Object obj);

    /**
     * Compares two tasks for equality.
     * 
     * @param task ITask to compare with.
     * @return <code>true</code> in case of fully equality by all attributes;
     *          <code>false</code> otherwise.
     */
    public abstract boolean equalsTask(ITask task);

    /**
     * Converts this <code>ITask</code> object to a <code>String</code>.
     * @return String representation of this object.
     * 
     */
    public abstract String toString();

    /**
     * @return  a hash code value for this object. 
     */
    public abstract int hashCode();    

}
