/**
 * Contains private realisations of tasks and events represented by public 
 * interfaces in package ru.amse.ksenofontova.jina.model.
 */
package ru.amse.ksenofontova.jina.model.impl;

import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;

import ru.amse.ksenofontova.jina.io.LoadException;
import ru.amse.ksenofontova.jina.io.TasksReader;
import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.model.IRepositopyManager;
import ru.amse.ksenofontova.jina.model.ITask;
import ru.amse.ksenofontova.jina.model.Identity;

/**
 * This class is representation of the <code>IRepositopyManager</code> 
 * that use <code>Collection<ITask></code> as a repository.
 * @see ru.amse.ksenofontova.jina.model.IRepositopyManager
 * @author Eugenia
 * 
 */
/* package */class CollectionRepositoryManager implements IRepositopyManager {
    
    private Collection<ITask> myTasks = new LinkedList<ITask>();    


    /**
     * Constructs CollectionRepositoryManager by reading tasks into it's repository 
     * (collection of <code>ITask</code>) by the given <code>TasksReader</code>
     * @param tasksReader to read tasks into repository 
     * @throws LoadException if there are some problems with reading tasks
     * (not enought memory)
     */
    public CollectionRepositoryManager(TasksReader tasksReader) throws LoadException {
        tasksReader.read(this);
    }

    /**
     * Constructs CollectionRepositoryManager with an empty repository 
     * (collection of <code>ITask</code>)
     */
    public CollectionRepositoryManager() {}
    

    /**
     * Removes specified <code>ITask</code> element from the repository.
     * @param task to be romoved
     */
    public void remove(ITask task) {
        myTasks.remove(task);
    }

    /**
     * Creates <code>ITask</code> element with 
     * <code>Identity.DAY_TASK</code> identity and specified parameters.
     * @param begin <code>Date</code> represents the beginning of the period
     * that task is planning for
     * @param end begin <code>Date</code> represents the completion of the period
     * that task is planning for
     * @param text task description
     */
    public ITask createDayTask(Date begin, Date end, String text) {
        ITask newTask = new Task(begin, end, text, Identity.DAY_TASK);
        return newTask;
    }

    /**
     * Adds <code>ITask</code> element to the repository.
     * @param task to be added.
     */
    public void add(ITask task) {
        myTasks.add(task);
    }

    /**
     * Returns Collection of <code>Task</code> where each element
     * fully or partly belongs to specified time interval or is a
     * permanent task that has been planned befor the Date specified
     * by the <code>end</code> parameter. Returnes an empty collection if there are no
     * required tasks. Returnes all the tasks if both parameters are null.
     * @param begin the beginning of the time interval. If it is null result
     * contains all the tasks that have been planned befor (not after) the 
     * Date specified by the <code>end</code> parameter.
     * @param end the ending of the time interval. If it is null result
     * contains all the tasks that have been planned from the Date specified
     * by the <code>begin</code> parameter.
     * @return Collection of <code>Task</code> where each element
     * fully or partly belongs to specified time interval or is a
     * permanent task that has been planned befor the Date specified
     * by the <code>end</code> parameter. Returnes an empty collection if there are no
     * required tasks. Returnes all the tasks if both parameters are null.
     */
    public Collection<ITask> getInRange(Date begin, Date end) {
        if((begin == null) && (end== null)) {
            return myTasks;
        }
        
        Collection<ITask> result = new LinkedList<ITask>();
        for (Iterator<ITask> iter = myTasks.iterator(); iter.hasNext();) {
            ITask task = iter.next();
            Date taskBegin = task.getBeginDate();
            Date taskEnd = task.getEndDate();
//          PERMANENT tasks can only have ExecutionState = IN_PROCESS
            // and occures only since the day they where planned.
            if(task.isPermanent()) {
                if(DateUtil.isInRange(taskBegin, null,end)) {
                    result.add(task);
                }
            } else if (DateUtil.isInRange(taskBegin, begin, end)
                || DateUtil.isInRange(taskEnd, begin, end)
                || (DateUtil.isInRange(taskBegin, null, end) && (DateUtil
                    .isInRange(taskEnd, end, null)))) {
                result.add(task);
            }

        }
        return result;

    }

}
