/**
 * Contains private realisations of tasks represented by public 
 * interfaces in package ru.amse.ksenofontova.jina.model.
 */
package ru.amse.ksenofontova.jina.model.impl;

import java.util.Date;

import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.model.ExecutionState;
import ru.amse.ksenofontova.jina.model.ITask;
import ru.amse.ksenofontova.jina.model.Identity;
import ru.amse.ksenofontova.jina.model.Priority;
import ru.amse.ksenofontova.jina.model.TimeLimitation;

/**
 * Represents ITask implementation (the planning stuff in common: 
 * both tasks and events).
 * 
 * @author Eugenia
 * 
 */
class Task implements ITask {    
    
    private Date         myBeginDate;
    private Date         myEndDate;
    private Identity       myIdentity       = DEFAULT_IDENTITY;
    private String         myText           = "";
    private Priority       myPriority       = DEFAULT_PROIRITY;
    private ExecutionState myExecutionState = DEFAULT_EXECUTION_STATE;
    private TimeLimitation myTimeLimitation = DEFAULT_TIME_LIMITATION;
    
    public Task(){
        myBeginDate = DateUtil.getToday();
        myEndDate = DateUtil.getToday();
    }

    /**
     * 
     * @param begin must be < end
     * @param end
     * @param text
     * @param ident
     * @throws IllegalArgumentException if some of data parameters 
     * is not instance of DateUtil or if begin >= end
     */
    Task(Date begin, Date end, String text, Identity ident)
        throws IllegalArgumentException {

        checkTimeLimit(begin, end);
        myBeginDate = begin;
        myEndDate = end;
        myIdentity = ident;
        myText = text;
    }

    /**
     * Checks correctness of time-limit represented by its begin and end
     * @param begin is the beginning of the checked time-limit
     * @param end is the end of the checked time-limit
     * @throws IllegalArgumentException if some of data parameters 
     * is not instance of DateUtil or if begin >= end
     */
    private void checkTimeLimit(Date begin, Date end)
        throws IllegalArgumentException {

        if (begin.after(end)) {
            throw new IllegalArgumentException(
                "Wrong time-limit parameters: begin >= end");
        }
    }

    /**
     * @return beginDate 
     */
    public Date getBeginDate() {
        return myBeginDate;
    }

    /**
     * @see ru.amse.ksenofontova.jina.model.ITask#getEndDate()
     */
    public Date getEndDate() {
        return myEndDate;
    }

    /* (non-Javadoc)
     * @see ru.amse.ksenofontova.jina.model.ITask#getIdentity()
     */
    public Identity getIdentity() {
        return myIdentity;
    }

    /* (non-Javadoc)
     * @see ru.amse.ksenofontova.jina.model.ITask#getPriority()
     */
    public Priority getPriority() {
        return myPriority;
    }

    /* (non-Javadoc)
     * @see ru.amse.ksenofontova.jina.model.ITask#getExecutionState()
     */
    public ExecutionState getExecutionState() {
        return myExecutionState;
    }

    /* (non-Javadoc)
     * @see ru.amse.ksenofontova.jina.model.ITask#getText()
     */
    public String getText() {
        return myText;
    }

    /*
     * (non-Javadoc)
     * 
     * @see ru.amse.ksenofontova.jina.model.ITask#isPermanent()
     */
    public boolean isPermanent() {
        return myTimeLimitation == TimeLimitation.PERMANENT;
    }

    public TimeLimitation getTimeLimitation() {
        return myTimeLimitation;
    }

    /**
     * 
     * @throws WrongTimeLimitException if endDate <= beginDate
     */
    public void setDates(Date beginDate, Date endDate) throws IllegalArgumentException {
        if(!isPermanent()) {
            checkTimeLimit(beginDate, endDate);
        }        
        myBeginDate = beginDate;
        myEndDate = endDate;
    }
        

    public void setIdentity(Identity ident) {
        myIdentity = ident;
    }

    /* (non-Javadoc)
     * @see ru.amse.ksenofontova.jina.model.ITask#setPriority(ru.amse.ksenofontova.jina.model.Priority)
     */
    public void setPriority(Priority prio) {
        myPriority = prio;
    }

    /* (non-Javadoc)
     * @see ru.amse.ksenofontova.jina.model.ITask#setExecutionState(ru.amse.ksenofontova.jina.model.ExecutionState)
     */
    public void setExecutionState(ExecutionState state) {
        myExecutionState = state;
    }

    /* (non-Javadoc)
     * @see ru.amse.ksenofontova.jina.model.ITask#setText(java.lang.String)
     */
    public void setText(String text) {
        myText = text;
    }

    /* (non-Javadoc)
     * @see ru.amse.ksenofontova.jina.model.ITask#setTimeLimitation(ru.amse.ksenofontova.jina.model.TimeLimitation)
     */
    public void setTimeLimitation(TimeLimitation timeLimitation) {
        myTimeLimitation = timeLimitation;
    }

    /**
     * Compares two objects for equality. The result is <code>true</code> if
     * and only if the argument is not <code>null</code> and is a instance of
     * <code>ITask</code> with equal attributes, as this object.
     * 
     * @param obj
     *        the object to compare with.
     * @return <code>true</code> if and only if the argument is not
     *         <code>null</code> and is a instance of <code>ITask</code>
     *         with equal attributes, as this object; <code>false</code>
     *         otherwise.
     */
    public boolean equals(Object obj) {
        return (obj instanceof ITask) && equalsTask((ITask) obj);
    }

    /**
     * Compares two tasks for equality.
     * 
     * @param task ITask to compare with.
     * @return <code>true</code> in case of fully equality by all attributes;
     *          <code>false</code> otherwise.
     */
    public boolean equalsTask(ITask task) {
        boolean result;
        if(task.getIdentity() == Identity.DAY_TASK) {
            result = (DateUtil.getDaysDiff(myBeginDate, task.getBeginDate()) == 0)
            && (DateUtil.getDaysDiff(myEndDate, task.getEndDate()) == 0)
            && (myIdentity.equals(task.getIdentity()))
            && (myText.equals(task.getText()))
            && (myPriority.equals(task.getPriority()))
            && (myExecutionState.equals(task.getExecutionState()))
            && (myTimeLimitation.equals(task.getTimeLimitation())); 
        } else {
            result = (DateUtil.getHoursDiff(myBeginDate, task.getBeginDate()) == 0)
            && (DateUtil.getHoursDiff(myEndDate, task.getEndDate()) == 0)
            && (myIdentity.equals(task.getIdentity()))
            && (myText.equals(task.getText()))
            && (myPriority.equals(task.getPriority()))
            && (myExecutionState.equals(task.getExecutionState()))
            && (myTimeLimitation.equals(task.getTimeLimitation()));
        }
        return result;
    }

    /**
     * Converts this <code>ITask</code> object to a <code>String</code> .
     * 
     */
    public String toString() {
        StringBuilder strB = new StringBuilder();
        strB.append(myIdentity);
        strB.append(" : ");
        strB.append(myBeginDate);
        strB.append(" - ");
        strB.append(myEndDate);
        strB.append(" : ");
        strB.append(myText);
        strB.append(" : ");
        strB.append(myPriority);
        strB.append(" : ");
        strB.append(myExecutionState);
        strB.append(" : ");
        strB.append(myTimeLimitation);
        return strB.toString();
    }

    /**
     * @return  a hash code value for this object. 
     */
    public int hashCode() {
        return myBeginDate.hashCode() + myEndDate.hashCode()
            + myIdentity.hashCode() + myText.hashCode() + myPriority.hashCode()
            + myExecutionState.hashCode() + myTimeLimitation.hashCode();

    }

}
