/**
 * 
 */
package ru.amse.ksenofontova.jina.view;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Shape;
import java.awt.geom.Area;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import java.util.Date;

import ru.amse.ksenofontova.jina.model.DateUtil;
import ru.amse.ksenofontova.jina.model.ITask;
import ru.amse.ksenofontova.jina.model.Priority;

/*package*/class TaskComponent {    
    /**
     * 
     */
    
    private static final String CANCELED_ICON_NAME = "canceled.gif";    
    private static final String DONE_ICON_NAME = "done.gif";    
    private static final String INPROCESS_ICON_NAME = "inprocess.gif";    
    private static final String IMPORTANT_ICON_NAME = "important.gif";
    
    private final TasksPanel myPanel;
    private final ITask      myTask;

    private final int        myX;
    private final int        myY;
    private final int        myWidth;
    private final int        myHeight = ViewParameters.getCellHeight()
                                      - ViewParameters.DEFAULT_BORDER_WIDTH;
    private final int        myMarginWidth = myHeight / 2;

    private static enum MarginType {
        CONTINUED, //for PERMANENT task
        ENDED, // for LIMITED task if its margin is in visible interval
        CUTTED; // for LIMITED task if its margin is out of visible interval
    }

    public TaskComponent(TasksPanel panel, ITask task, int y) {
        myPanel = panel;
        myTask = task;
        myY = y;

        Date firstVisibleDate = ViewParameters.getFirstVisibleDate();

        // setting myX
        int beginDayOffset = DateUtil.getDaysDiff(task.getBeginDate(),
            firstVisibleDate);
        int xBeginCoordIndex = beginDayOffset < 0 ? 0 : beginDayOffset;
        myX = myPanel.xCoordinate(xBeginCoordIndex) 
            + 2 * ViewParameters.DEFAULT_BORDER_WIDTH;

        // setting myWidth
        int endDayOffset;
        int scale = ViewParameters.getScale();
        if (task.isPermanent()) {
            endDayOffset = scale - 1;
        } else {
            endDayOffset = DateUtil.getDaysDiff(task.getEndDate(),
                firstVisibleDate);
        }
        int widthInDays = (endDayOffset >= scale) ? scale - beginDayOffset
            : endDayOffset + 1 - beginDayOffset;

        int xBegin = myX;
        int xEnd = myPanel.xCoordinate(beginDayOffset + widthInDays);
        myWidth = xEnd - xBegin - 2 * ViewParameters.DEFAULT_BORDER_WIDTH;
    }

    protected void paint(Graphics2D g2D) {
        g2D.setStroke(new BasicStroke(ViewParameters.DEFAULT_BORDER_WIDTH));
        drawShape(g2D);
        drawText(g2D);
    }

    private Color getTextColor() {
        if (myPanel.getSelectedTask() == myTask) {
            return ViewParameters.SelectedTaskFontColor;
        }
        return ViewParameters.TaskFontColor;
    }

    private Color getBackgroundColor() {
        if (myPanel.getSelectedTask() == myTask) {
            return ViewParameters.SelectedTaskBackgroundColor;
        }
        return ViewParameters.TaskBackgroundColor;
    }

    private MarginType getLeftMarginType() {
        MarginType leftMarginType;
        if (myTask.isPermanent()) {
            leftMarginType = MarginType.CONTINUED;
        } else {
            int beginDayOffset = DateUtil.getDaysDiff(myTask.getBeginDate(),
                ViewParameters.getFirstVisibleDate());
            leftMarginType = (beginDayOffset < 0) ? MarginType.CUTTED
                : MarginType.ENDED;
        }
        return leftMarginType;
    }

    private MarginType getRightMarginType() {
        MarginType rightMarginType;
        if (myTask.isPermanent()) {
            rightMarginType = MarginType.CONTINUED;
        } else {
            int endDayOffset = DateUtil.getDaysDiff(myTask.getEndDate(),
                ViewParameters.getFirstVisibleDate());
            rightMarginType = (endDayOffset >= ViewParameters.getScale())
                ? MarginType.CUTTED : MarginType.ENDED;
        }
        return rightMarginType;
    }

    private Shape createMargin(int x, MarginType type) {
        switch (type){
        case CONTINUED:
            return createArrowPoligon(x, myY, myMarginWidth * 2, myHeight);
        case ENDED:
            return new RoundRectangle2D.Double(x, myY, myMarginWidth * 2,
                myHeight, myHeight / 2, myHeight / 2);
        case CUTTED:
            return new Rectangle2D.Double(x, myY, myMarginWidth * 2, myHeight);
        default:
            return new RoundRectangle2D.Double(x, myY, myMarginWidth * 2,
                myHeight, myHeight / 2, myHeight / 2);
        }
    }

    private void drawShape(Graphics2D g2D) {
        Area area = new Area();
        Area leftMargin = new Area(createMargin(myX, getLeftMarginType()));

        Area center = new Area(new Rectangle2D.Double(myX + myMarginWidth, myY,
            myWidth - 2 * myMarginWidth, myHeight));

        Area rightMargin = new Area(createMargin(myX + myWidth - 2
            * myMarginWidth, getRightMarginType()));

        area.add(leftMargin);
        area.add(center);
        area.add(rightMargin);

        g2D.setColor(getBackgroundColor());
        g2D.fill(area);

        g2D.setColor(ViewParameters.TaskBorderColor);
        g2D.draw(area);
    }

    // Creates arrow poligon shape inscribed into the frame specified by given parameters
    private static Shape createArrowPoligon(int x, int y, int width, int height) {
        GeneralPath s = new GeneralPath();
        s.moveTo(x, y);
        s.lineTo((x + width / 4), y + height / 2);
        s.lineTo(x, y + height);
        s.lineTo(x + width * 3 / 4, y + height);
        s.lineTo(x + width, y + height / 2);
        s.lineTo(x + width * 3 / 4, y);
        s.closePath();
        return s;
    }

    // TODO make it nice! 
    // May be create filed to increase the speed of ToolipTaext grawing???
    protected String getToolTipText() {
        StringBuilder text = new StringBuilder();
        if (!myTask.isPermanent()) {
            Date beginDate = myTask.getBeginDate();
            Date endDate = myTask.getEndDate();
            text.append(DateUtil.getDateString(beginDate));
            if (!beginDate.equals(endDate)) {
                text.append(" - ");
                text.append(DateUtil.getDateString(endDate));
            }            
        } else {
            text.append("Permanent");
        }
        text.append("    ");
        if(myTask.getPriority() == Priority.IMPORTANT) {
            text.append("Important    "); 
        }
        switch (myTask.getExecutionState()){
        case IN_PROCESS:
            text.append("In process"); break;
        case DONE:
            text.append("Done"); break;
        case CANCELED:
            text.append("Canceled"); break;
        }
        return text.toString();
    }
    
    
    private void drawText(Graphics2D g2D) {
        if (myWidth < 2 * myMarginWidth) {
            return;
        }
        // drawing icons
        int drawingMargin = myMarginWidth;
        if (myTask.getPriority() == Priority.IMPORTANT) {
            Image importantImage = ImageUtil.createImage(IMPORTANT_ICON_NAME);
            g2D.drawImage(importantImage, myX + drawingMargin, myY
                + (myHeight - importantImage.getHeight(myPanel)) / 2
                + ViewParameters.DEFAULT_BORDER_WIDTH, getBackgroundColor(), myPanel);
            
            drawingMargin += myMarginWidth / 2
                + importantImage.getWidth(myPanel);
        }

        Image executionStateImage;
        switch (myTask.getExecutionState()){
        case IN_PROCESS:
            executionStateImage = ImageUtil.createImage(INPROCESS_ICON_NAME);
            break;
        case DONE:
            executionStateImage = ImageUtil.createImage(DONE_ICON_NAME);
            break;
        case CANCELED:
            executionStateImage = ImageUtil.createImage(CANCELED_ICON_NAME);
            break;
        default:
            executionStateImage = ImageUtil.createImage(DONE_ICON_NAME);
        }
        g2D.drawImage(executionStateImage, myX + drawingMargin, myY
            + (myHeight - executionStateImage.getHeight(myPanel)) / 2
            + ViewParameters.DEFAULT_BORDER_WIDTH, getBackgroundColor(), myPanel);
        
        drawingMargin += myMarginWidth / 2
            + executionStateImage.getWidth(myPanel);

        g2D.setFont(ViewParameters.getTaskFont());
        g2D.setColor(getTextColor());
        myPanel.drawText(g2D, myTask.getText(), myX + drawingMargin, myY
            + ViewParameters.DEFAULT_BORDER_WIDTH, myWidth - myMarginWidth
            - drawingMargin, myHeight);
    }

    /**
     * @param x
     * @param y
     * @return
     */
    public boolean containsPosition(int x, int y) {
        return ((x > myX) && (x < myX + myWidth) && (y > myY) && (y < myY
            + myHeight));
    }

    public ITask getTask() {
        return myTask;
    }

}
